#include "tableprint.hpp"
#include <iomanip>
#include <algorithm>
#include <locale>
#include <cwchar>

tableprint::tableprint() {
    // Set locale for wide character support
    std::setlocale(LC_ALL, "");
}

tableprint::~tableprint() {}

void tableprint::add_row(const std::vector<std::string>& row) {
    rows.push_back(row);
}

void tableprint::print() {
    if (rows.empty()) return;

    // Calculate column widths
    std::vector<size_t> col_widths(rows[0].size(), 0);
    for (const auto& row : rows) {
        for (size_t i = 0; i < row.size(); ++i) {
            col_widths[i] = std::max(col_widths[i], row[i].length());
        }
    }

    // Print top border
    std::wcout << L"\033[37m"; // Light grey color for borders
    std::wcout << L"┌";
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::wcout << std::wstring(col_widths[i] + 2, L'─');
        if (i < rows[0].size() - 1) std::wcout << L"┬";
    }
    std::wcout << L"┐" << std::endl;

    // Print header
    std::wcout << L"│";
    std::wcout << L"\033[1;36m"; // Cyan color for header
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::wcout << L" " << std::setw(col_widths[i]) << std::left << std::wstring(rows[0][i].begin(), rows[0][i].end()) << L" ";
        if (i < rows[0].size() - 1) {
            std::wcout << L"\033[37m│\033[1;36m"; // Border color then back to cyan
        } else {
            std::wcout << L"\033[37m│"; // Just border color for last column
        }
    }
    std::wcout << std::endl;

    // Print header separator
    std::wcout << L"├";
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::wcout << std::wstring(col_widths[i] + 2, L'─');
        if (i < rows[0].size() - 1) std::wcout << L"┼";
    }
    std::wcout << L"┤" << std::endl;

    // Print rows
    for (size_t row_idx = 1; row_idx < rows.size(); ++row_idx) {
        const auto& row = rows[row_idx];
        std::wcout << L"│";
        for (size_t i = 0; i < row.size(); ++i) {
            std::string cell = row[i];
            // Replace :tick: and :cross: with colored symbols
            if (cell == ":tick:") {
                std::wcout << L" " << L"\033[1;32m" << std::setw(col_widths[i]) << std::left << L"✓" << L"\033[37m" << L" │";
            } else if (cell == ":cross:") {
                std::wcout << L" " << L"\033[1;31m" << std::setw(col_widths[i]) << std::left << L"✗" << L"\033[37m" << L" │";
            } else {
                std::wcout << L" " << std::setw(col_widths[i]) << std::left << std::wstring(cell.begin(), cell.end()) << L" │";
            }
        }
        std::wcout << std::endl;
    }

    // Print bottom border
    std::wcout << L"└";
    for (size_t i = 0; i < rows[0].size(); ++i) {
        std::wcout << std::wstring(col_widths[i] + 2, L'─');
        if (i < rows[0].size() - 1) std::wcout << L"┴";
    }
    std::wcout << L"┘" << L"\033[0m" << std::endl; // Reset color
} 